@@ -26,13 +26,16 @@ Copyright 2017-2018 David Anderson. All rights reserved.
 */
 
 #include <config.h>
-
-#include <stdio.h>
+#include <stdio.h> /* FILE decl for dd_esb.h, printf etc */
 #include <string.h>
 #include <stdlib.h>
+
 #include "dwarf.h"
 #include "libdwarf.h"
 #include "libdwarf_private.h"
+#include "dd_defined_types.h"
+#include "dd_checkutil.h"
+#include "dd_glflags.h"
 #include "dd_globals.h"
 #include "dd_naming.h"
 #include "dd_sanitized.h"
